/*
 * Copyright 2008 jpp Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sourceforge.jpp;

import java.lang.annotation.Annotation;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.ServiceLoader;
import java.util.Set;
import javax.annotation.processing.AbstractProcessor;
import javax.annotation.processing.ProcessingEnvironment;
import javax.annotation.processing.RoundEnvironment;
import javax.annotation.processing.SupportedSourceVersion;
import javax.lang.model.SourceVersion;
import javax.lang.model.element.TypeElement;
import javax.lang.model.util.ElementFilter;

@SupportedSourceVersion(SourceVersion.RELEASE_6)
public class PatternProcessor extends AbstractProcessor {

    private final Map<String, PatternHandler> _patternHandlers = new HashMap<String, PatternHandler>();

    @Override
    public Set<String> getSupportedAnnotationTypes() {
        return _patternHandlers.keySet();
    } 

    @Override public Set<String> getSupportedOptions()
    {
        Set<String> options = new HashSet<String>();
        
        for(PatternHandler handler : _patternHandlers.values() )
        {
            options.addAll(handler.getSupportedOptions());
        }
        return options;
    }
    
    @Override
    public boolean process(Set<? extends TypeElement> annotations, RoundEnvironment roundEnv) {

        for (TypeElement annotation : annotations) {

            PatternHandler handler = _patternHandlers.get(annotation.getQualifiedName().toString());

            if (handler != null) {
                for (TypeElement type : ElementFilter.typesIn(roundEnv.getElementsAnnotatedWith(annotation)) ) {

                    handler.handlePattern(type);
                }
            }
        }
        return true;
    }

    @Override
    public synchronized void init(ProcessingEnvironment env) {
        super.init(env);
          
        ServiceLoader<PatternHandler> loader = ServiceLoader.load(PatternHandler.class,getClass().getClassLoader());
        
        for(PatternHandler handler: loader)
        {
            handler.init(processingEnv);
            registerPatternHandler(handler);
        }
    }

    public <A extends Annotation> void registerPatternHandler(PatternHandler<A> handler) {
        _patternHandlers.put(handler.getPatternAnnotationType().getName().toString(), handler);
    }
}
