/*
 * Copyright 2008 jpp Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sourceforge.jpp.handler;


import javax.lang.model.element.ElementKind;
import javax.lang.model.element.TypeElement;
import net.sourceforge.jpp.annotation.Adapter;
import net.sourceforge.jpp.model.AdapterData;

public class AdapterHandler extends SourceGeneratorPatternHandler<Adapter> {

    public AdapterHandler() {
        super(Adapter.class);
    }

    @Override
    public AdapterData buildModel(TypeElement type) {
        
        Adapter annotation = type.getAnnotation(Adapter.class);
        AdapterData model = new AdapterData();
        
        model.getTarget().setName(newTypeName(annotation.name()));
        model.getSource().setName(newTypeName(type));
        model.setSpec( newTypeName(annotation.spec()) );
        
        model.setImpl(annotation.impl().length() == 0 ? null : annotation.impl());
        model.setFactory(annotation.factory());
        model.setReference(annotation.reference());
        
        model.setConcrete(annotation.concrete() && (model.getImpl() != null || model.isFactory()));
        model.setThreaded(annotation.threaded());
        
        TypeElement spec = processingEnv.getElementUtils().getTypeElement(annotation.spec());
        
        ElementKind kind = spec.getKind();
        model.setExtends(kind == ElementKind.CLASS);
        model.setImplements(kind == ElementKind.INTERFACE);
        model.setMethods(listMethods(spec));

        return model;
    }

}
