/*
 * Copyright 2008 jpp Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sourceforge.jpp.handler;

import javax.lang.model.element.TypeElement;
import net.sourceforge.jpp.annotation.Factory;
import net.sourceforge.jpp.model.FactoryData;
import net.sourceforge.jpp.model.TypeName;

public class FactoryHandler extends SourceGeneratorPatternHandler<Factory> {

    public FactoryHandler() {
        super(Factory.class);
    }

    @Override
    public FactoryData buildModel(TypeElement type) {
        
        Factory annotation = type.getAnnotation(Factory.class);
        FactoryData model = new FactoryData();
        
        model.getTarget().setName(newTypeName(annotation.name(), type, annotation.NAME_FORMAT));
        model.getSource().setName(new TypeName(type.getQualifiedName().toString()));
        model.setSpec( new TypeName(annotation.spec().length() == 0 ? model.getSource().getName().getQualifiedName() : annotation.spec()) );
        model.setConcrete(annotation.concrete());
        model.setThreaded(annotation.threaded());
        model.setInit(getInit(type));

        return model;
    }
    
}
