/*
 * Copyright 2008 jpp Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sourceforge.jpp.handler;

import javax.lang.model.element.ElementKind;
import javax.lang.model.element.Modifier;
import javax.lang.model.element.TypeElement;
import net.sourceforge.jpp.annotation.Singleton;
import net.sourceforge.jpp.model.SingletonData;
import net.sourceforge.jpp.model.TypeName;

public class SingletonHandler extends SourceGeneratorPatternHandler<Singleton> {

    public SingletonHandler() {
        super(Singleton.class);
    }

    @Override
    public SingletonData buildModel(TypeElement type) {


        ElementKind kind = type.getKind();
        Singleton annotation = type.getAnnotation(Singleton.class);
        SingletonData model = new SingletonData();

        model.getTarget().setName(newTypeName(annotation.name(), type, annotation.NAME_FORMAT));
        model.getSource().setName(new TypeName(type.getQualifiedName().toString()));
        model.setSpec(new TypeName(annotation.spec().length() == 0 ? model.getSource().getName().getQualifiedName() : annotation.spec()));

        model.setLazy(annotation.lazy());
        model.setExtends(kind == ElementKind.CLASS);
        model.setImplements(kind == ElementKind.INTERFACE);
        model.setConcrete(kind != ElementKind.INTERFACE && !type.getModifiers().contains(Modifier.ABSTRACT));
        model.setThreaded(annotation.threaded());
        model.setLoader(annotation.loader() || !model.isConcrete());
        model.setInit(getInit(type));

        return model;
    }
}
