/*
 * Copyright 2008 jpp Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sourceforge.jpp.handler;

import javax.lang.model.element.ElementKind;
import javax.lang.model.element.TypeElement;
import net.sourceforge.jpp.annotation.Wrapper;
import net.sourceforge.jpp.model.WrapperData;

public class WrapperHandler extends SourceGeneratorPatternHandler<Wrapper> {

    public WrapperHandler() {
        super(Wrapper.class);
    }

    @Override
    public WrapperData buildModel(TypeElement type) {
        
        Wrapper annotation = type.getAnnotation(Wrapper.class);
        WrapperData model = new WrapperData();
        
        model.getTarget().setName(newTypeName(annotation.name(), type, annotation.NAME_FORMAT));
        model.getSource().setName(newTypeName(type));
        model.setSpec( annotation.spec().length() == 0 ? model.getSource().getName() : newTypeName(annotation.spec()) );
        model.setConcrete(annotation.concrete());
        model.setThreaded(annotation.threaded());
        
        ElementKind kind = type.getKind();
        model.setExtends(kind == ElementKind.CLASS);
        model.setImplements(kind == ElementKind.INTERFACE);
        model.setMethods(listMethods(type));

        return model;
    }
    

}
