/*
 * Copyright 2008 jpp Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sourceforge.jpp.model;

import java.util.HashSet;
import java.util.Set;
import javax.lang.model.element.Modifier;

public class ModifierData {

    private final Set<Modifier> _modifiers;
    
    public ModifierData(Set<Modifier> modifiers)
    {
        _modifiers = modifiers;
    }
    
    public ModifierData(Modifier... modifiers)
    {
        _modifiers = new HashSet<Modifier>();
        for(Modifier m : modifiers)
        {
            _modifiers.add(m);
        }
    }
    
    public boolean isAbstract()
    {
        return _modifiers.contains(Modifier.ABSTRACT);
    }
    
    public boolean isFinal()
    {
        return _modifiers.contains(Modifier.FINAL);
    }
    
    public boolean isNative()
    {
        return _modifiers.contains(Modifier.NATIVE);
    }
    
    public boolean isPrivate()
    {
        return _modifiers.contains(Modifier.PRIVATE);
    }
            
    public boolean isProtected()
    {
        return _modifiers.contains(Modifier.PROTECTED);
    }
    
    public boolean isPublic()
    {
        return _modifiers.contains(Modifier.PUBLIC);
    }
    
    public boolean isStatic()
    {
        return _modifiers.contains(Modifier.STATIC);
    }
    
    public boolean isStrictfp()
    {
        return _modifiers.contains(Modifier.STRICTFP);
    }

    public boolean isSynchronized()
    {
        return _modifiers.contains(Modifier.SYNCHRONIZED);
    }
    
    public boolean isTransient()
    {
        return _modifiers.contains(Modifier.TRANSIENT);
    }
    
    public boolean isVolatile()
    {
        return _modifiers.contains(Modifier.VOLATILE);
    }

    public Set<Modifier> getAll()
    {
        return _modifiers;
    }
    
    public Modifier getAccess()
    {
        return _modifiers.contains(Modifier.PUBLIC) ? Modifier.PUBLIC :
            _modifiers.contains(Modifier.PROTECTED) ? Modifier.PROTECTED :
                _modifiers.contains(Modifier.PRIVATE) ? Modifier.PRIVATE :
                    null;
    }
}

