/*
 * Copyright 2008 jpp Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sourceforge.jpp.model;

public class TypeName
{
    public static final String ARRAY_SUFFIX ="[]";
    private String _simpleName;
    private String _qualifiedName;
    private String _qualifier;
    private String _parameters;
    private String _qualifiedBaseName;
    private String _simpleBaseName;
    private boolean _array;
    
    public TypeName(String name, String parameters, boolean array)
    {
        setup(name, parameters, array);
    }
    
    public TypeName(String name, String parameters)
    {
        setup(name, parameters, false);
    }
    
    public TypeName(String name)
    {
        int len = name.length();
        int aidx = len > 2 && name.charAt(len-2) == '[' && name.charAt(len-1) == ']' ? len - 2 : len;
        int pidx = name.indexOf('<');
        
        boolean array = aidx != len;
        
        if ( pidx <= 0 )
        {
            setup(name.substring(0, aidx), null, array);
        }
        else
        {
            setup(name.substring(0, pidx), name.substring(pidx,aidx), array);
        }
    }
    
    protected void setup(String name, String parameters, boolean array)
    {
        _array = array;
        _qualifiedBaseName = name;
        int dot = name.lastIndexOf('.');
        if ( dot <= 0 )
        {
            _qualifier = null;
            _simpleBaseName = name;
        }
        else
        {
            _qualifier = name.substring(0,dot);
            _simpleBaseName = name.substring(dot + 1);
        }
        
        _parameters = parameters;
        if ( parameters == null)
        {
            _qualifiedName = _qualifiedBaseName;
            _simpleName = _simpleBaseName;
        }
        else
        {
            _simpleName = _simpleBaseName + parameters;
            _qualifiedName = _qualifiedBaseName + parameters;
        }
        
        if ( array )
        {
            _simpleName+=ARRAY_SUFFIX;
            _qualifiedName+=ARRAY_SUFFIX;
        }
    }
    
    public String getSimpleName()
    {
        return _simpleName;
    }
    
    public String getSimpleBaseName()
    {
        return _simpleBaseName;
    }
    
    public String getQualifiedName()
    {
        return _qualifiedName;
    }
    
    public String getQualifier()
    {
        return _qualifier;
    }
    
    public String getParameters()
    {
        return _parameters;
    }
    
    public String getQualifiedBaseName()
    {
        return _qualifiedBaseName;
    }
    
    public boolean isArray()
    {
        return _array;
    }
}
